/**
 * @fileOverview
 * @author spikelinyu
 */

var util  = require('../util');

const chai = require('chai');
const expect = chai.expect;
const sinon = require('sinon');

describe('util', ()=> {

  describe('#frameworkCdnPath()', ()=> {
    it('should get assets', (done)=> {
      util.frameworkCdnPath({
        "pi-kissy": "0.x.x",
        "m-base": "2.x.x"
      }).then(function(result){
        expect(result).to.be.a('object');
        expect(result.js).to.be.a('object');
        expect(result.css).to.be.a('object');
        expect(result.js['g.alicdn.com']).to.be.a('array');
        done();
      })
    })
  });

  describe('#assetsCdnPath()', ()=> {
    it('should get assets', (done)=> {
      util.assetsCdnPath({
        "kimi":"6.x.x",
        "require":"0.x.x",
        "mmd":"6.x.x",
        "kissy-mod-define":"0.x.x"
      }).then(function(result){
        done();
      })
    })
  });

  describe('#fizXtplName()', ()=> {
    it('should be xtpl name', (done)=> {
      var name = util.fizXtplName('mod.xtpl','pc');
      expect(name).to.eql('mod_pc.xtpl');
      done();
    })
  });

  describe('#comboAssets()', ()=> {
    it('should be combo assets', (done)=> {
      var comboAssets = util.comboAssets(['kimi','mmd','kissy-mod-define'],{
        kimi:{
          js: [ '//g.alicdn.com??/kissy/kimi/6.0.1/kimi-min.js'],
          css: []
        },
        mmd:{
          js: [ '//g.alicdn.com??/kissy/mmd/6.3.0/mmd-min.js' ],
          css: []
        },
        "kissy-mod-define":{
          js: [ '//g.alicdn.com??/mtb/3rd/0.0.5/kissy-mod-define-min.js' ],
          css: []
        }
      });
      expect(comboAssets).to.be.a('object');
      expect(comboAssets.js).to.be.a('object');
      expect(comboAssets.css).to.be.a('object');
      expect(comboAssets.js['g.alicdn.com']).to.be.a('string');
      expect(comboAssets.js['g.alicdn.com']).to.eql('//g.alicdn.com??/kissy/kimi/6.0.1/kimi-min.js,/kissy/mmd/6.3.0/mmd-min.js,/mtb/3rd/0.0.5/kissy-mod-define-min.js');
      done();
    })
  });

});
